#!/bin/sh

ONIE_AUTOCFG_CACHE=/var/cache/onieseed

onieseed_log() {
	logger -t onieseed "$@"
}

# Borrowed from Debian's initramfs infrastructure
# Resolve device node from a name.  This expands any LABEL or UUID.
# $1=name
# Resolved name is echoed.
resolve_device() {
	local DEV="$1"

	case $DEV in
	LABEL=*)
		DEV="${DEV#LABEL=}"

		# support any / in LABEL= path (escape to \x2f)
		case "${DEV}" in
		*/*)
		if command -v sed >/dev/null 2>&1; then
			DEV="$(echo ${DEV} | sed 's,/,\\x2f,g')"
		else
			if [ "${DEV}" != "${DEV#/}" ]; then
				DEV="\x2f${DEV#/}"
			fi
			if [ "${DEV}" != "${DEV%/}" ]; then
				DEV="${DEV%/}\x2f"
			fi
			IFS='/'
			newroot=
			for s in $DEV; do
				newroot="${newroot:+${newroot}\\x2f}${s}"
			done
			unset IFS
			DEV="${newroot}"
		fi
		esac
		DEV="/dev/disk/by-label/${DEV}"
		;;
	UUID=*)
		DEV="/dev/disk/by-uuid/${DEV#UUID=}"
		;;
	MTD=*)
		DEV="${DEV#MTD=}"
		;;
	*)
		DEV=""
		;;
	esac
	# Only canonicalise if a valid file, in case $DEV isn't a filename
	[ -e "$DEV" ] && DEV=$(readlink -f "$DEV") || DEV=""
	echo "$DEV"
}

config_net_interface() {
	local udevrule="/etc/udev/rules.d/000_net-onie.rules"

	# if not all variables are provided don't try and configure
	# let netcfg figure it out if it can
	if [ -z "$NETDEV" -o -z "$NETMAC" -o -z "$NETSYS" -o -z "$NETDRV" ]; then
		onieseed_log "NET: not all required values were provided."
		return 0
	fi

	{
		/bin/echo -n "ACTION==\"add\", ";
		/bin/echo -n "KERNELS==\"${NETDEV}\", ";
		/bin/echo -n "SUBSYSTEMS==\"${NETSYS}\", ";
		/bin/echo -n "DRIVERS==\"${NETDRV}\", ";
		/bin/echo -n "NAME=\"eth0\", ";
		/bin/echo "RUN+=\"/etc/hotplug.d/net/hw-detect.setmac ${NETMAC}\"";
	} >"$udevrule"
	# NOTE: Make sure the NAME=eth0 above matches the interface used in the
	# preseed file
	onieseed_log "NET: configured."
}

# Figure out which installer we are using and then configure it
# for auto installation
# $1 - is used to pass the /dev/ node entry where the installer should
#      be written to
config_bootloader() {
	local status=/var/lib/dpkg/status
	local f=/tmp/onieseed.boot
	local dev="$1"

	rm -f "$f"

	if grep "^Package:\s*grub-installer$" "$status" >/dev/null 2>&1; then
		#Setup GRUB:
		{
			echo "d-i grub-installer/only_debian boolean true"
			echo "d-i grub-installer/with_other_os boolean true"
			[ -n "$dev" ] && echo "d-i grub-installer/bootdev string $dev"
			echo "d-i lilo-installer/skip boolean true"
		} >"$f"
		onieseed_log "BOOTLOADER: configured for grub."
	elif grep "^Package:\s*u-boot-installer$" "$status" >/dev/null 2>&1; then
		onieseed_log "BOOTLOADER: TODO: add some configuration for u-boot installer"
	else
		onieseed_log "BOOTLOADER: unknown installer being used, no autoconfig done"
	fi

	if [ -e "$f" ]; then
		preseed_location file://${f}
		rm -f "$f"
	fi
}

bootloader_write_recovery() {
	local dev="$1"
	local mnt="/tmp/onieseed-recovery"
	local recovery_failed=no
	local recovery="${mnt}/recovery"

	# mount partition
	mkdir -p "$mnt"
	mount "$dev" "$mnt" || return 1

	# look for executable left by embedded-installer, if exists execute it
	# the standard is to chroot into <c-i partition>/recovery and
	# execute <c-i parition>/recovery/recovery-setup - no arguments to the
	# script are listened to
	if test -x "${recovery}/recovery-setup" ; then
		if [ ! -f "${recovery}/proc/cmdline" ]; then
			mkdir -p "${recovery}/proc"
			mount -t proc proc "${recovery}/proc"
		fi
		if [ ! -d "${recovery}/sys/devices" ]; then
			mkdir -p "${recovery}/sys"
			mount -t sysfs sysfs "${recovery}/sys"
		fi
		mount -o bind /dev "${recovery}/dev"
		chroot "${recovery}" /recovery-setup || recovery_failed=yes
	else
		logger -t recover-install "warning: no recovery-setup script provide by embedded-installer"
	fi

	# umount partition
	umount "${recovery}/proc" || true
	umount "${recovery}/sys"  || true
	umount "${recovery}/dev"  || true
	umount "$mnt" || true
	rmdir "$mnt" || true
	if test "yes" = "$recovery_failed"; then
		return 1
	fi
	return 0
}

# Mount temporary embedded installer partition
ei_partition_mnt="/tmp/ei-partition"
_mount_ei_part() {
	local dev="$1"

	mkdir -p "$ei_partition_mnt"
	log-output -t onieseed mount "$dev" "$ei_partition_mnt" || {
		onieseed_log "ERROR: Unable to mount embedded installer dir"
		return 1
	}
	return 0
}

_umount_ei_part() {
	umount "$ei_partition_mnt"
	rmdir "$ei_partition_mnt"
}

onieseed_copy_apt() {
	# mount and copy deb tarball for bootstrap
	local dev="$1"
	local REPO_ARCHIVE="/repo-archive"

	mkdir -p "$REPO_ARCHIVE"
	_mount_ei_part "$dev" || return $?

	if [ -e "${ei_partition_mnt}/repo-archive.tar" ] ; then
		tar -C "$REPO_ARCHIVE" -xf "${ei_partition_mnt}/repo-archive.tar"
	else
		onieseed_log "warning: Unable to find repo-archive.tar"
	fi

	_umount_ei_part
}

onieseed_copy_preseed() {
	# mount and copy out user preseed if it exists
	local dev="$1"
	local output="$2"

	# This is early, so make sure file system support is loaded.
	# More than likely the file system support is already compiled
	# into the kernel.
	local fstype=$(blkid -o value -s TYPE "$dev")
	onieseed_log "Loading file system module: $fstype"
	modprobe "$fstype"
	_mount_ei_part "$dev" || return $?

	if [ -e "${ei_partition_mnt}/user_preseed" ] ; then
		onieseed_log "Found user preseed..."
		cp "${ei_partition_mnt}/user_preseed" "$output"
	fi

	_umount_ei_part
}
